@@ -5,12 +5,13 @@ from .notSynchronizedException import NotSynchronizedException
 from .notConnectedException import NotConnectedException
 from .synchronizationListener import SynchronizationListener
 from .reconnectListener import ReconnectListener
-from ..models import MetatraderHistoryOrders, MetatraderDeals, date, random_id
+from ..models import MetatraderHistoryOrders, MetatraderDeals, date, random_id, MetatraderSymbolSpecification, \
+    MetatraderTradeResponse, MetatraderSymbolPrice, MetatraderAccountInformation, MetatraderPosition, MetatraderOrder
 import socketio
 import asyncio
 import re
 from datetime import datetime
-from typing import Coroutine
+from typing import Coroutine, List
 import pytz
 
 
@@ -30,6 +31,8 @@ class MetaApiWebsocketClient:
         Args:
             token: Authorization token.
             domain: Domain to connect to, default is agiliumtrade.agiliumtrade.ai.
+            request_timeout: Timeout for socket requests in seconds.
+            connect_timeout: Timeout for connecting to server in seconds.
         """
         self._url = f'https://mt-client-api-v1.{domain}'
         self._request_timeout = request_timeout
@@ -140,7 +143,7 @@ class MetaApiWebsocketClient:
             self._requestResolves = {}
             self._synchronizationListeners = {}
 
-    async def get_account_information(self, account_id: str) -> asyncio.Future:
+    async def get_account_information(self, account_id: str) -> 'asyncio.Future[MetatraderAccountInformation]':
         """Returns account information for a specified MetaTrader account
         (see https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readAccountInformation/).
 
@@ -153,7 +156,7 @@ class MetaApiWebsocketClient:
         response = await self._rpc_request(account_id, {'type': 'getAccountInformation'})
         return response['accountInformation']
 
-    async def get_positions(self, account_id: str) -> asyncio.Future:
+    async def get_positions(self, account_id: str) -> 'asyncio.Future[List[MetatraderPosition]]':
         """Returns positions for a specified MetaTrader account
         (see https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readPositions/).
 
@@ -166,7 +169,7 @@ class MetaApiWebsocketClient:
         response = await self._rpc_request(account_id, {'type': 'getPositions'})
         return response['positions']
 
-    async def get_position(self, account_id: str, position_id: str) -> asyncio.Future:
+    async def get_position(self, account_id: str, position_id: str) -> 'asyncio.Future[MetatraderPosition]':
         """Returns specific position for a MetaTrader account
         (see https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readPosition/).
 
@@ -180,7 +183,7 @@ class MetaApiWebsocketClient:
         response = await self._rpc_request(account_id, {'type': 'getPosition', 'positionId': position_id})
         return response['position']
 
-    async def get_orders(self, account_id: str) -> asyncio.Future:
+    async def get_orders(self, account_id: str) -> 'asyncio.Future[List[MetatraderOrder]]':
         """Returns open orders for a specified MetaTrader account
         (see https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readOrders/).
 
@@ -193,7 +196,7 @@ class MetaApiWebsocketClient:
         response = await self._rpc_request(account_id, {'type': 'getOrders'})
         return response['orders']
 
-    async def get_order(self, account_id: str, order_id: str) -> asyncio.Future:
+    async def get_order(self, account_id: str, order_id: str) -> 'asyncio.Future[MetatraderOrder]':
         """Returns specific open order for a MetaTrader account
         (see https://metaapi.cloud/docs/client/websocket/api/readTradingTerminalState/readOrder/).
 
@@ -336,7 +339,7 @@ class MetaApiWebsocketClient:
         """
         return self._rpc_request(account_id, {'type': 'removeHistory'})
 
-    async def trade(self, account_id: str, trade) -> asyncio.Future:
+    async def trade(self, account_id: str, trade) -> 'asyncio.Future[MetatraderTradeResponse]':
         """Execute a trade on a connected MetaTrader account
         (see https://metaapi.cloud/docs/client/websocket/api/trade/).
 
@@ -348,19 +351,21 @@ class MetaApiWebsocketClient:
             A coroutine resolving with trade result.
 
         Raises:
-            TradeException: On trade error.
+            TradeException: On trade error, check error properties for error code details.
         """
         response = await self._rpc_request(account_id, {'type': 'trade', 'trade': trade})
+        if 'response' not in response:
+            response['response'] = {}
         if 'stringCode' not in response['response']:
             response['response']['stringCode'] = response['response']['description']
+        if 'numericCode' not in response['response']:
+            response['response']['numericCode'] = response['response']['error']
         if response['response']['stringCode'] in ['ERR_NO_ERROR', 'TRADE_RETCODE_PLACED', 'TRADE_RETCODE_DONE',
                                                   'TRADE_RETCODE_DONE_PARTIAL', 'TRADE_RETCODE_NO_CHANGES']:
-            if 'numericCode' not in response['response']:
-                response['response']['numericCode'] = response['response']['error']
             return response['response']
         else:
-            raise TradeException(response['response']['message'], response['response']['error'],
-                                 response['response']['description'])
+            raise TradeException(response['response']['message'], response['response']['numericCode'],
+                                 response['response']['stringCode'])
 
     async def subscribe(self, account_id: str):
         """Subscribes to the Metatrader terminal events
@@ -376,8 +381,8 @@ class MetaApiWebsocketClient:
             try:
                 await self._rpc_request(account_id, {'type': 'subscribe'})
             except Exception as err:
-                print(f'[{datetime.now().isoformat()}] MetaApi websocket client failed to receive subscribe response',
-                      err)
+                print(f'[{datetime.now().isoformat()}] MetaApi websocket client failed to receive subscribe response '
+                      f'(this usually does not mean an error)', err)
 
         asyncio.create_task(run_subscribe())
 
@@ -425,7 +430,8 @@ class MetaApiWebsocketClient:
         """
         return self._rpc_request(account_id, {'type': 'subscribeToMarketData', 'symbol': symbol})
 
-    async def get_symbol_specification(self, account_id: str, symbol: str) -> asyncio.Future:
+    async def get_symbol_specification(self, account_id: str, symbol: str) -> \
+            'asyncio.Future[MetatraderSymbolSpecification]':
         """Retrieves specification for a symbol
         (see https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/getSymbolSpecification/).
 
@@ -439,7 +445,7 @@ class MetaApiWebsocketClient:
         response = await self._rpc_request(account_id, {'type': 'getSymbolSpecification', 'symbol': symbol})
         return response['specification']
 
-    async def get_symbol_price(self, account_id: str, symbol: str) -> asyncio.Future:
+    async def get_symbol_price(self, account_id: str, symbol: str) -> 'asyncio.Future[MetatraderSymbolPrice]':
         """Retrieves price for a symbol
         (see https://metaapi.cloud/docs/client/websocket/api/retrieveMarketData/getSymbolPrice/).
 
@@ -533,7 +539,11 @@ class MetaApiWebsocketClient:
         self._requestResolves[request_id] = asyncio.Future()
         request['accountId'] = account_id
         await self._socket.emit('request', request)
-        resolve = await asyncio.wait_for(self._requestResolves[request_id], timeout=self._request_timeout)
+        try:
+            resolve = await asyncio.wait_for(self._requestResolves[request_id], timeout=self._request_timeout)
+        except asyncio.TimeoutError:
+            raise TimeoutException(f"MetaApi websocket client request {request['requestId']} of type "
+                                   f"{request['type']} timed out")
         return resolve
 
     def _convert_error(self, data) -> Exception:
@@ -568,156 +578,301 @@ class MetaApiWebsocketClient:
     async def _process_synchronization_packet(self, data):
         try:
             if data['type'] == 'authenticated':
+                on_connected_tasks: List[asyncio.Task] = []
+
+                async def run_on_connected(listener):
+                    try:
+                        await listener.on_connected()
+                    except Exception as err:
+                        print('Failed to notify listener about connected event', err)
+
                 if data['accountId'] in self._synchronizationListeners:
                     for listener in self._synchronizationListeners[data['accountId']]:
-                        try:
-                            await listener.on_connected()
-                        except Exception as err:
-                            print('Failed to notify listener about connected event', err)
+                        on_connected_tasks.append(asyncio.create_task(run_on_connected(listener)))
+                if len(on_connected_tasks) > 0:
+                    await asyncio.wait(on_connected_tasks)
             elif data['type'] == 'disconnected':
+                on_disconnected_tasks: List[asyncio.Task] = []
+
+                async def run_on_disconnected(listener):
+                    try:
+                        await listener.on_disconnected()
+                    except Exception as err:
+                        print('Failed to notify listener about disconnected event', err)
+
                 if data['accountId'] in self._synchronizationListeners:
                     for listener in self._synchronizationListeners[data['accountId']]:
-                        try:
-                            await listener.on_disconnected()
-                        except Exception as err:
-                            print('Failed to notify listener about disconnected event', err)
+                        on_disconnected_tasks.append(asyncio.create_task(run_on_disconnected(listener)))
+                if len(on_disconnected_tasks) > 0:
+                    await asyncio.wait(on_disconnected_tasks)
             elif data['type'] == 'accountInformation':
                 if data['accountInformation'] and (data['accountId'] in self._synchronizationListeners):
-                    for listener in self._synchronizationListeners[data['accountId']]:
+                    on_account_information_updated_tasks: List[asyncio.Task] = []
+
+                    async def run_on_account_info(listener):
                         try:
                             await listener.on_account_information_updated(data['accountInformation'])
                         except Exception as err:
                             print('Failed to notify listener about accountInformation event', err)
+
+                    for listener in self._synchronizationListeners[data['accountId']]:
+                        on_account_information_updated_tasks.append(asyncio.create_task(run_on_account_info(listener)))
+                    if len(on_account_information_updated_tasks) > 0:
+                        await asyncio.wait(on_account_information_updated_tasks)
             elif data['type'] == 'deals':
                 if 'deals' in data:
                     for deal in data['deals']:
+                        on_deal_added_tasks: List[asyncio.Task] = []
+
+                        async def run_on_deal_added(listener):
+                            try:
+                                await listener.on_deal_added(deal)
+                            except Exception as err:
+                                print('Failed to notify listener about deals event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_deal_added(deal)
-                                except Exception as err:
-                                    print('Failed to notify listener about deals event', err)
+                                on_deal_added_tasks.append(asyncio.create_task(run_on_deal_added(listener)))
+                        if len(on_deal_added_tasks) > 0:
+                            await asyncio.wait(on_deal_added_tasks)
             elif data['type'] == 'orders':
                 if 'orders' in data:
                     for order in data['orders']:
+                        on_order_updated_tasks: List[asyncio.Task] = []
+
+                        async def run_on_order_updated(listener):
+                            try:
+                                await listener.on_order_updated(order)
+                            except Exception as err:
+                                print('Failed to notify listener about orders event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_order_updated(order)
-                                except Exception as err:
-                                    print('Failed to notify listener about orders event', err)
+                                on_order_updated_tasks.append(asyncio.create_task(run_on_order_updated(listener)))
+                        if len(on_order_updated_tasks) > 0:
+                            await asyncio.wait(on_order_updated_tasks)
             elif data['type'] == 'historyOrders':
                 if 'historyOrders' in data:
                     for historyOrder in data['historyOrders']:
+                        on_history_order_added_tasks: List[asyncio.Task] = []
+
+                        async def run_on_order_added(listener):
+                            try:
+                                await listener.on_history_order_added(historyOrder)
+                            except Exception as err:
+                                print('Failed to notify listener about historyOrders event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_history_order_added(historyOrder)
-                                except Exception as err:
-                                    print('Failed to notify listener about historyOrders event', err)
+                                on_history_order_added_tasks.append(asyncio.create_task(run_on_order_added(listener)))
+                        if len(on_history_order_added_tasks) > 0:
+                            await asyncio.wait(on_history_order_added_tasks)
             elif data['type'] == 'positions':
                 if 'positions' in data:
                     for position in data['positions']:
+                        on_position_updated_tasks: List[asyncio.Task] = []
+
+                        async def run_on_position_updated(listener):
+                            try:
+                                await listener.on_position_updated(position)
+                            except Exception as err:
+                                print('Failed to notify listener about positions event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_position_updated(position)
-                                except Exception as err:
-                                    print('Failed to notify listener about positions event', err)
+                                on_position_updated_tasks.append(asyncio.create_task(run_on_position_updated(listener)))
+                        if len(on_position_updated_tasks) > 0:
+                            await asyncio.wait(on_position_updated_tasks)
             elif data['type'] == 'update':
                 if 'accountInformation' in data and (data['accountId'] in self._synchronizationListeners):
-                    for listener in self._synchronizationListeners[data['accountId']]:
+                    on_account_information_updated_tasks: List[asyncio.Task] = []
+
+                    async def run_on_account_information_updated(listener):
                         try:
                             await listener.on_account_information_updated(data['accountInformation'])
                         except Exception as err:
                             print('Failed to notify listener about update event', err)
+
+                    for listener in self._synchronizationListeners[data['accountId']]:
+                        on_account_information_updated_tasks.append(
+                            asyncio.create_task(run_on_account_information_updated(listener)))
+                    if len(on_account_information_updated_tasks) > 0:
+                        await asyncio.wait(on_account_information_updated_tasks)
                 if 'updatedPositions' in data:
                     for position in data['updatedPositions']:
+                        on_position_updated_tasks: List[asyncio.Task] = []
+
+                        async def run_on_position_updated(listener):
+                            try:
+                                await listener.on_position_updated(position)
+                            except Exception as err:
+                                print('Failed to notify listener about update event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_position_updated(position)
-                                except Exception as err:
-                                    print('Failed to notify listener about update event', err)
+                                on_position_updated_tasks.append(
+                                    asyncio.create_task(run_on_position_updated(listener)))
+                        if len(on_position_updated_tasks) > 0:
+                            await asyncio.wait(on_position_updated_tasks)
                 if 'removedPositionIds' in data:
                     for positionId in data['removedPositionIds']:
+                        on_position_removed_tasks: List[asyncio.Task] = []
+
+                        async def run_on_position_removed(listener):
+                            try:
+                                await listener.on_position_removed(positionId)
+                            except Exception as err:
+                                print('Failed to notify listener about update event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_position_removed(positionId)
-                                except Exception as err:
-                                    print('Failed to notify listener about update event', err)
+                                on_position_removed_tasks.append(
+                                    asyncio.create_task(run_on_position_removed(listener)))
+                        if len(on_position_removed_tasks) > 0:
+                            await asyncio.wait(on_position_removed_tasks)
                 if 'updatedOrders' in data:
                     for order in data['updatedOrders']:
+                        on_order_updated_tasks: List[asyncio.Task] = []
+
+                        async def run_on_order_updated(listener):
+                            try:
+                                await listener.on_order_updated(order)
+                            except Exception as err:
+                                print('Failed to notify listener about update event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_order_updated(order)
-                                except Exception as err:
-                                    print('Failed to notify listener about update event', err)
+                                on_order_updated_tasks.append(
+                                    asyncio.create_task(run_on_order_updated(listener)))
+                        if len(on_order_updated_tasks) > 0:
+                            await asyncio.wait(on_order_updated_tasks)
                 if 'completedOrderIds' in data:
                     for orderId in data['completedOrderIds']:
+                        on_order_completed_tasks: List[asyncio.Task] = []
+
+                        async def run_on_order_completed(listener):
+                            try:
+                                await listener.on_order_completed(orderId)
+                            except Exception as err:
+                                print('Failed to notify listener about update event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_order_completed(orderId)
-                                except Exception as err:
-                                    print('Failed to notify listener about update event', err)
+                                on_order_completed_tasks.append(
+                                    asyncio.create_task(run_on_order_completed(listener)))
+                        if len(on_order_completed_tasks) > 0:
+                            await asyncio.wait(on_order_completed_tasks)
                 if 'historyOrders' in data:
                     for historyOrder in data['historyOrders']:
+                        on_history_order_added_tasks: List[asyncio.Task] = []
+
+                        async def run_on_history_order_added(listener):
+                            try:
+                                await listener.on_history_order_added(historyOrder)
+                            except Exception as err:
+                                print('Failed to notify listener about update event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_history_order_added(historyOrder)
-                                except Exception as err:
-                                    print('Failed to notify listener about update event', err)
+                                on_history_order_added_tasks.append(
+                                    asyncio.create_task(run_on_history_order_added(listener)))
+                        if len(on_history_order_added_tasks) > 0:
+                            await asyncio.wait(on_history_order_added_tasks)
                 if 'deals' in data:
                     for deal in data['deals']:
+                        on_deal_added_tasks: List[asyncio.Task] = []
+
+                        async def run_on_deal_added(listener):
+                            try:
+                                await listener.on_deal_added(deal)
+                            except Exception as err:
+                                print('Failed to notify listener about update event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_deal_added(deal)
-                                except Exception as err:
-                                    print('Failed to notify listener about update event', err)
+                                on_deal_added_tasks.append(
+                                    asyncio.create_task(run_on_deal_added(listener)))
+                        if len(on_deal_added_tasks) > 0:
+                            await asyncio.wait(on_deal_added_tasks)
             elif data['type'] == 'dealSynchronizationFinished':
                 if data['accountId'] in self._synchronizationListeners:
-                    for listener in self._synchronizationListeners[data['accountId']]:
+                    on_deal_synchronization_finished_tasks: List[asyncio.Task] = []
+
+                    async def run_on_deal_synchronization_finished(listener):
                         try:
                             await listener.on_deal_synchronization_finished(data['synchronizationId'])
                         except Exception as err:
                             print('Failed to notify listener about dealSynchronizationFinished event', err)
+
+                    for listener in self._synchronizationListeners[data['accountId']]:
+                        on_deal_synchronization_finished_tasks.append(
+                                    asyncio.create_task(run_on_deal_synchronization_finished(listener)))
+                    if len(on_deal_synchronization_finished_tasks) > 0:
+                        await asyncio.wait(on_deal_synchronization_finished_tasks)
             elif data['type'] == 'orderSynchronizationFinished':
                 if data['accountId'] in self._synchronizationListeners:
-                    for listener in self._synchronizationListeners[data['accountId']]:
+                    on_order_synchronization_finished_tasks: List[asyncio.Task] = []
+
+                    async def run_on_order_synchronization_finished(listener):
                         try:
                             await listener.on_order_synchronization_finished(data['synchronizationId'])
                         except Exception as err:
                             print('Failed to notify listener about orderSynchronizationFinished event', err)
+
+                    for listener in self._synchronizationListeners[data['accountId']]:
+                        on_order_synchronization_finished_tasks.append(
+                            asyncio.create_task(run_on_order_synchronization_finished(listener)))
+                    if len(on_order_synchronization_finished_tasks) > 0:
+                        await asyncio.wait(on_order_synchronization_finished_tasks)
             elif data['type'] == 'status':
                 if data['accountId'] in self._synchronizationListeners:
-                    for listener in self._synchronizationListeners[data['accountId']]:
+                    on_broker_connection_status_changed_tasks: List[asyncio.Task] = []
+
+                    async def run_on_broker_connection_status_changed(listener):
                         try:
                             await listener.on_broker_connection_status_changed(bool(data['connected']))
                         except Exception as err:
                             print('Failed to notify listener about brokerConnectionStatusChanged event', err)
+
+                    for listener in self._synchronizationListeners[data['accountId']]:
+                        on_broker_connection_status_changed_tasks.append(
+                            asyncio.create_task(run_on_broker_connection_status_changed(listener)))
+                    if len(on_broker_connection_status_changed_tasks) > 0:
+                        await asyncio.wait(on_broker_connection_status_changed_tasks)
             elif data['type'] == 'specifications':
                 if 'specifications' in data:
                     for specification in data['specifications']:
+                        on_symbol_specification_updated_tasks: List[asyncio.Task] = []
+
+                        async def run_on_symbol_specification_updated(listener):
+                            try:
+                                await listener.on_symbol_specification_updated(specification)
+                            except Exception as err:
+                                print('Failed to notify listener about specifications event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_symbol_specification_updated(specification)
-                                except Exception as err:
-                                    print('Failed to notify listener about specifications event', err)
+                                on_symbol_specification_updated_tasks.append(
+                                    asyncio.create_task(run_on_symbol_specification_updated(listener)))
+                            if len(on_symbol_specification_updated_tasks) > 0:
+                                await asyncio.wait(on_symbol_specification_updated_tasks)
             elif data['type'] == 'prices':
                 if 'prices' in data:
                     for price in data['prices']:
+                        on_symbol_price_updated_tasks: List[asyncio.Task] = []
+
+                        async def run_on_symbol_price_updated(listener):
+                            try:
+                                await listener.on_symbol_price_updated(price)
+                            except Exception as err:
+                                print('Failed to notify listener about prices event', err)
+
                         if data['accountId'] in self._synchronizationListeners:
                             for listener in self._synchronizationListeners[data['accountId']]:
-                                try:
-                                    await listener.on_symbol_price_updated(price)
-                                except Exception as err:
-                                    print('Failed to notify listener about prices event', err)
+                                on_symbol_price_updated_tasks.append(
+                                    asyncio.create_task(run_on_symbol_price_updated(listener)))
+                            if len(on_symbol_price_updated_tasks) > 0:
+                                await asyncio.wait(on_symbol_price_updated_tasks)
         except Exception as err:
             print('Failed to process incoming synchronization packet', err)
 
