@@ -65,11 +65,11 @@ class TestMetatraderAccountApi:
     async def test_retrive_mt_accounts(self):
         """Should retrieve MT accounts."""
         client.get_accounts = AsyncMock(return_value=[{'_id': 'id'}])
-        accounts = await api.get_accounts('profileId')
+        accounts = await api.get_accounts({'provisioningProfileId': 'profileId'})
         assert list(map(lambda a: a.id, accounts)) == ['id']
         for account in accounts:
             assert isinstance(account, MetatraderAccount)
-        client.get_accounts.assert_called_with('profileId')
+        client.get_accounts.assert_called_with({'provisioningProfileId': 'profileId'})
 
     @pytest.mark.asyncio
     async def test_retrive_mt_account_by_id(self):
@@ -599,12 +599,14 @@ class TestMetatraderAccountApi:
         """Should connect to an MT terminal."""
         websocket_client.add_synchronization_listener = MagicMock()
         websocket_client.subscribe = AsyncMock()
+        MetaApiConnection.initialize = AsyncMock()
         client.get_account = AsyncMock(return_value={'_id': 'id', 'synchronizationMode': 'user'})
         account = await api.get_account('id')
-        storage = MockStorage()
+        storage = MockStorage('accountId')
         connection = await account.connect(storage)
         assert isinstance(connection, MetaApiConnection)
         assert connection.history_storage == storage
+        connection.initialize.assert_called()
         websocket_client.add_synchronization_listener.assert_called_with('id', storage)
         websocket_client.subscribe.assert_called_with('id')
 
